require 'Color.rb'

class BookmarkManager
	def initialize( editControl, markerAllocator )
		@editControl = editControl
		@markerType = markerAllocator.newMarkerType()
		@markerType.define( :circle, Color::Teal, Color::Aqua )
		@bookmarks = Array.new
	end
	
  def toggleBookmark()
    bookmark = bookmarkAt( @editControl.currentLineNumber() )
    if bookmark
      @editControl.markerDeleteHandle( bookmark )
      @bookmarks.delete( bookmark )
    else
      @bookmarks  <<  @markerType.newMarker( @editControl.currentLineNumber() )
    end
  end

  def moveToNext()
		line = @markerType.cyclingNextMarker( @editControl.currentLineNumber()+1 )
    @editControl.gotoLine( line )
  end

  def moveToPrevious()
		line = @markerType.cyclingPreviousMarker( @editControl.currentLineNumber()-1 )
    @editControl.gotoLine( line )
  end

  def moveToLastAdded()
    line = @editControl.markerLineFromHandle( @bookmarks.last() )
    @editControl.gotoLine( line )
  end

  def hasBookmark?()
    !@bookmarks.empty?()
  end

	# private
  def bookmarkAt( line )
    @bookmarks.each do |bookmark|
      return bookmark if @editControl.markerLineFromHandle( bookmark ) == line
    end
    return nil
  end
	
end

