require 'scintilla/ScConstant.rb'
require 'Color'

class Console
  attr_reader :editControl
  
  def initialize( editControl )
    @editControl = editControl
    setUpEditControl()
  end
  
  def to_s()
    "Console<%s>" % @editControl
  end
  
  def setUpEditControl()
    @editControl.bufferedDraw = true
    
    @editControl.wrapMode = Scintilla::Constant::SC_WRAP_WORD
    
    @editControl.readOnly = true
    
		setStyle( Scintilla::Constant::STYLE_DEFAULT, 
              Color::Black, 
              Color::White, 
              11, 
              "Courier New" )
    @editControl.styleSetBold( Scintilla::Constant::STYLE_DEFAULT, true )
    
    @defaultStyle = 0
    @commandStyle = 1
    setStyle( @commandStyle, Color::Blue )
    
    @successStyle = 2
    setStyle( @successStyle, Color::Black, Color::Lime )
    
    @successInformationStyle = 3
    setStyle( @successInformationStyle, Color::DarkGray, Color::Lime )
    
    @failureStyle = 4
    setStyle( @failureStyle, Color::White, Color::Red )
    
    @failureInfomrationStyle = 5
    setStyle( @failureInfomrationStyle, Color::OffWhite, Color::Red )
    
    @informationStyle = 6
    setStyle( @informationStyle, Color::Navy )
    
    @failureMessageStyle = 7
    setStyle( @failureMessageStyle, Color::Red )
  end
		
  def setStyle( style, fore, back = Color::White, size = nil, face = nil )
    @editControl.styleSetFore( style, fore )
    @editControl.styleSetBack( style, back )
		@editControl.styleSetSize( style, size ) if size
		@editControl.styleSetFont( style, face ) if face
  end

  def clearOutput()
    @editControl.readOnly = false
    @editControl.clearAll()
    @editControl.readOnly = true
  end
  
  def captureCommandOutput( command, workingDirectory =nil )
    Dir.chdir( workingDirectory ) if workingDirectory
    clearOutput()
    addStyledText( command + "\n", @commandStyle )
    
    output = `#{command} 2>&1`    # redirect stderr to stdout for capture

    addAndStyleText( output )
    @editControl.documentEnd()
  end
  
  def addStyledText( text, style )
    return unless text.length() > 0
    @editControl.readOnly = false
    pos = @editControl.currentPos()
    @editControl.addText( text )
    @editControl.startStyling( pos, -1 )
    @editControl.setStyling( text.length(), style )
    @editControl.readOnly = true
  end
  
  def addAndStyleText( text )
    lines = text.split( "\n" )
    lines.each do |line|
      case line
        when /^Loaded suite/
          style = @informationStyle
        # Failure occurred in testRemoveListener(TestObservable) 
        when /Failure occurred in\s+(\w+)\(([^\)]+)\)\s+\[([^\]]+)\](.*)/
          markFailureLocation( $2, $1, $3 ) 
          style = @failureMessageStyle
        # Error occurred in testRemoveListener(TestObservable):
        when /Error occurred in\s+(\w+)\(([^\)]+)\)(.*)/
          style = @failureMessageStyle
        #  5 runs, 11 assertions, 0 failures, 0 errors
        when /^\s*(\d+)\s+runs\D*(\d+)\s+assertions\D*(\d+)\s+failures\D*(\d+)\s+errors/
          printRunStatistics( $1.to_i(), $2.to_i(), $3.to_i(), $4.to_i() )
          next
        else 
          style = @defaultStyle
      end
      addStyledText( line + "\n", style )
    end
  end
  
  def printRunStatistics( runs, assertions, failures, errors )
    wasSucessful = failures != 0  ||  errors != 0
    styles = wasSucessful ? [@failureStyle, @failureInfomrationStyle] : 
                            [@successStyle, @successInformationStyle]
    separator = ''
    [ [runs , 'runs', true], 
      [assertions, 'assertions', true], 
      [failures, 'failures', false], 
      [errors, 'errors', false ] ].each do |data|
      next if data[0] == 0  &&  !data[2]
      addStyledText( separator, styles[0] )
      addStyledText( data[0].to_s(), styles[ data[0] == 0 ? 0 : 1 ] )
      addStyledText( ' ' + data[1].to_s(), styles[0] )
      separator = (data[1] == 'assertions') ? ' ==> ' : ', '
    end
  end
  
  def markFailureLocation( fixtureName, testName, location )
  end

  def canCopy?()
    @editControl.canCopy?()
  end
  
  def copy()
    @editControl.copy()
  end
  
end








