# FileDialog

module FileDialog


# extension:
# def FileDialog.doFileDialog( open
#                              filters, 
#                              defaultExtension, 
#                              fileName, 
#                              hideReadOnly, 
#                              multiSelect, 
#                              mustExist, 
#                              createPrompt, 
#                              overWritePrompt )

  # filters are an array of pair: [  [Filter Name, FilerMask], ... ]
  # => [ [ 'Ruby Files (*.rb)', '*.rb' ], [ 'All Files (*.*)', '*.*' ] ],

  def FileDialog.asWin32Filters( filters )
    filterString = ''
	for filter in filters
	  filterString << filter.join( '|' ) << '|'
	end
	filterString << ( filterString.empty? ? '||' : '|' )
  end


  # Open a file dialog to select one file for opening
  # Returns: 
  #   nil if user canceled
  #   fileName: String if user select a file
  def FileDialog.open( filters  ='', 
                       defaultExtension = '', 
                       fileName = '', 
                       flags = [] )
    files = FileDialog.doFileDialog( true,
                                     asWin32Filters( filters ),
                                     defaultExtension,
                                     fileName,
                                     flags.include?( :hideReadOnly ),
                                     false,
                                     flags.include?( :mustExist ),
                                     flags.include?( :createPrompt ),
                                     false )
    files ? files[0] : nil
  end

  # Open a file dialog to select multiple files for opening
  # Returns:
  #   nil if user canceled
  #   [fileNames]: Array of string if user selected some files
  def FileDialog.openMultiSelect( filters = '', 
                                  defaultExtension = '', 
                                  fileName = '', 
                                  flags = [] )
    FileDialog.doFileDialog( true,
                             asWin32Filters( filters ),
                             defaultExtension,
                             fileName,
                             flags.include?( :hideReadOnly ),
                             true,
                             flags.include?( :mustExist ),
                             flags.include?( :createPrompt ),
                             false )
  end

  # Open a file dialog to select a file for writing
  # Returns:
  #   nil if user canceled
  #   fileName: String if user selected a fileName
  def FileDialog.save( filters = '', 
                       defaultExtension = '', 
                       fileName = '', 
                       flags = [] )
    FileDialog.doFileDialog( false,
                             asWin32Filters( filters ),
                             defaultExtension,
                             fileName,
                             flags.include?( :hideReadOnly ),
                             false,
                             false,
                             false,
                             flags.include?( :overWritePrompt ) )
  end


end