require 'TraceDebug.rb'
require 'scintilla/ScConstant.rb'

class FoldManipulator
	def initialize( sourceDocument, folderMargin )
		@sourceDocument = sourceDocument
		@editControl = @sourceDocument.editControl()
		@folderMargin = folderMargin
	end

  def marginFoldAction( modifiers, position, margin )
		clickedLineNumber = @editControl.lineFromPosition( position )
		TraceDebug.puts "Toggling fold on line: %d, modifiers: %s" % 
        [clickedLineNumber, modifiers]
=begin
    dumpFolders() if modifiers.controlPressed?()
    if modifiers.shiftPressed?()
      toggleCompactFoldOnCurrentLevel( clickedLineNumber )
    els
=end    
    if margin == @folderMargin  and  clickedLineNumber >= 0
      toggleFolder( clickedLineNumber )
		end
  end
	
	def toggleFolder( lineNumber )
		@editControl.toggleFold( lineNumber )
	end

	def foldAll()
  end
  
  def foldPoint?( lineNumber )
    safeFoldData( lineNumber ) & 
        Scintilla::Constant::SC_FOLDLEVELHEADERFLAG != 0
  end
  
  def blankLine?( lineNumber )
    safeFoldData( lineNumber ) & Scintilla::Constant::SC_FOLDLEVELWHITEFLAG != 0
  end

  # need to substract SC_FOLDLEVELBASE ?
  def foldLevel( lineNumber )
    safeFoldData( lineNumber ) & Scintilla::Constant::SC_FOLDLEVELNUMBERMASK
  end
      
  def safeFoldData( lineNumber )
    ensureStyledToLine( lineNumber )
    @editControl.foldLevel( lineNumber )
  end
      
  def ensureStyledTo( pos )
    endStyledPos = @editControl.endStyled()
    # work around folding bug: does not create fold points correctly if asking 
    # to style one line at a time
    @editControl.colourise( endStyledPos, @editControl.length() ) if endStyledPos < pos
#    @editControl.colourise( endStyledPos, pos ) if endStyledPos < pos
#    TraceDebug.puts "Styling from %d to %d." % [endStyledPos,pos] if endStyledPos < pos
  end
  
  def ensureStyledToLine( line )
    ensureStyledTo( @editControl.lineEndPosition( line ) )
  end
  
  def dumpFolders( lineNumberFrom =0, lineNumberTo =-1)
    lineNumberTo = @editControl.lineCount() if lineNumberTo == -1
    TraceDebug.puts "----- Dumping folder structure from line %d to %d -----" % 
        [lineNumberFrom, lineNumberTo]
    for lineNumber in lineNumberFrom...lineNumberTo
      flag = foldPoint?( lineNumber ) ? 'F' : ' '
      flag += blankLine?( lineNumber ) ? 'W' : ' '
      #TraceDebug.puts "%s %3x : %s <===> %s" % [flag, 
      TraceDebug.puts "%s %3x : %s" % [flag, 
          foldLevel(lineNumber), 
          @editControl.getLine(lineNumber).inspect().gsub( /^"|"$|\\n/, '' ) ]
      #@editControl.styledTextLineRange(lineNumber,lineNumber+1)[1].join(',') ]
    end
    TraceDebug.puts "--------------------End Dumping -----------------------" 
  end
  
  def toggleCompactFoldOnCurrentLevel( startingLineNumber )
    for lineNumber in startingLineNumber..@editControl.lineCount()
      if foldPoint?(lineNumber )
        @editControl.showLines( lineNumber, lineNumber )
        @editControl.setFoldExpanded( lineNumber, false )
      else
        @editControl.hideLines( lineNumber, lineNumber )
      end
    end
  end
    
end




