require 'scintilla/ScConstant.rb'

class MarkerAllocator
	def initialize( editControl )
		@editControl = editControl
		@nextAvailableMarker = 0
	end
	
	def newMarkerType()
		id = @nextAvailableMarker
		@nextAvailableMarker += 1
		MarkerType.new( id, @editControl )
	end
end


class MarkerType
	attr_reader :id, :mask
	
	@@scintillaMarkerType = {
    :circle => Scintilla::Constant::SC_MARK_CIRCLE,
    :roundRect => Scintilla::Constant::SC_MARK_ROUNDRECT,
    :arrow => Scintilla::Constant::SC_MARK_ARROW,
    :smallRect => Scintilla::Constant::SC_MARK_SMALLRECT,
    :shortArrow => Scintilla::Constant::SC_MARK_SHORTARROW,
    :empty => Scintilla::Constant::SC_MARK_EMPTY,
    :arrowDown => Scintilla::Constant::SC_MARK_ARROWDOWN,
    :minus => Scintilla::Constant::SC_MARK_MINUS,
    :plus => Scintilla::Constant::SC_MARK_PLUS,
    :vLine => Scintilla::Constant::SC_MARK_VLINE,
    :lCorner => Scintilla::Constant::SC_MARK_LCORNER,
    :tCorner => Scintilla::Constant::SC_MARK_TCORNER,
    :boxPlus => Scintilla::Constant::SC_MARK_BOXPLUS,
    :boxPlusConnected => Scintilla::Constant::SC_MARK_BOXPLUSCONNECTED,
    :boxMinus => Scintilla::Constant::SC_MARK_BOXMINUS,
    :boxMinusConnected => Scintilla::Constant::SC_MARK_BOXMINUSCONNECTED,
    :lCornerCurve => Scintilla::Constant::SC_MARK_LCORNERCURVE,
    :tCornerCurve => Scintilla::Constant::SC_MARK_TCORNERCURVE,
    :circlePlus => Scintilla::Constant::SC_MARK_CIRCLEPLUS,
    :circlePlusConnected => Scintilla::Constant::SC_MARK_CIRCLEPLUSCONNECTED,
    :circleMinus => Scintilla::Constant::SC_MARK_CIRCLEMINUS,
    :circleMinusConnected => Scintilla::Constant::SC_MARK_CIRCLEMINUSCONNECTED,
    :background => Scintilla::Constant::SC_MARK_BACKGROUND
	}
	
	def initialize( id, editControl )
		@id = id
		@mask = 1 << id
		@editControl = editControl
	end
	
	def define( shape=nil, fore =nil, back =nil )
		self.shape = shape if shape
		self.foreColor = fore if fore
		self.backColor = back if back
	end
	
	def shape=( shape )
		shapeId = @@scintillaMarkerType[shape]
		@editControl.markerDefine( @id, shapeId )
	end
	
	def foreColor=( fore )
		@editControl.markerSetFore( @id, fore )
	end
	
	def backColor=( back )
		@editControl.markerSetBack( @id, back )
	end
	
	def newMarker( lineNumber )
		@editControl.markerAdd( lineNumber, @id )
	end

	# Returns:
	# line number where the marker is found
	# -1 if there no marker of this type
	def nextMarker( startingLineNumber )
    startingLineNumber >= 0 ? @editControl.markerNext( startingLineNumber, @mask ) : -1
	end
	
	def cyclingNextMarker( startingLineNumber )
    line = nextMarker( startingLineNumber )
		line = nextMarker( 0 ) if line < 0
		line
	end
	
	def previousMarker( startingLineNumber )
		startingLineNumber >= 0 ? @editControl.markerPrevious( startingLineNumber, @mask ) : -1
	end
	
	def cyclingPreviousMarker( startingLineNumber )
		line = previousMarker( startingLineNumber )
		line = previousMarker( @editControl.lineCount() ) if line < 0
		line
	end
end
	

