# Smart indent

require 'scintilla/ScConstant.rb'


class Array
  def indexNotOf( object )
    index =0
    while index < length()
      if at(index) != object
        break;
	  end
	  index = index+1
	end
    return index
  end
end


class SmartIndent
	def initialize( sourceDocument )
		@source = sourceDocument
		@editControl = @source.editControl()
	end
	
  # changes the indentation of the previous and current lines
  def smartIndent()
    currentLine = @editControl.currentLineNumber()
		if currentLine > 0
		  previousLineNumber = currentLine-1
			previousLineKeyWord = firstWordThatIsKeyWordAndNoEndOnLine( previousLineNumber )
			if shouldUnindentLine( previousLineNumber )
        if shouldIndentLine( previousLineNumber )
          changeLineIndentation( previousLineNumber, 0, false )       
        else
			    unindentLine( previousLineNumber, false )
        end
			elsif shouldIndentLine( previousLineNumber )
			  indentLine( previousLineNumber, false )
			end
		end
		
		if shouldIndentLine( currentLine )
			indentLine( currentLine, true )
		else
			changeLineIndentation( currentLine, 0, true )
		end
  end

  # indicates that the line should be unindented if it contains either 
	# 'end' or 'else' and no other key words.
  def shouldUnindentLine( lineNumber )
		firstKeyWord = firstWordThatIsKeyWordAndNoEndOnLine( lineNumber )
		%w( end else elsif rescue).include?( firstKeyWord )
  end

	# indicates that the line should be indented if the previous line starts with 
	# 'class, def, for...' and does not contains 'end'
  def shouldIndentLine( lineNumber )
		previousNonBlankLine = findNonBlankLineBefore( lineNumber )
		firstKeyWord = firstWordThatIsKeyWordAndNoEndOnLine( previousNonBlankLine )
		%w( module class def for while do if else begin 
        elsif rescue case when).include?( firstKeyWord )
  end
  
  # decrease the indentation level by one in comparison to the previous non blank line
  def unindentLine( lineNumber, updateCaretPosition )
    changeLineIndentation( lineNumber, -1, updateCaretPosition )
  end
  
  # increase the indentation level by one in comparison to the previous non blank line
  def indentLine( lineNumber, updateCaretPosition )
    changeLineIndentation( lineNumber, 1, updateCaretPosition )
  end

  # returns the indentation of the first non blank line found before lineNumber. 0 if none is found.  
  def previousNonBlankLineIndentation( lineNumber )
    while lineNumber > 0
      lineNumber -= 1
			return @editControl.lineIndentation( lineNumber ) if !blankLine?( lineNumber )
    end
    return 0
  end
	
	# returns the line number of the first line before lineNumber which is not a blank line, -1 if none
	def findNonBlankLineBefore( lineNumber )
		while lineNumber >= 0
			lineNumber -= 1
			return lineNumber if !blankLine?( lineNumber )
		end
		return -1
	end
  
  def blankLine?( lineNumber )
    return @editControl.getLine( lineNumber ) =~ /^\s*$/
  end
  
  # increase, decrease  the indentation level by one in comparison to the previous non blank line,
  # or copy the indentation level
  # direction: 0=> copy indentation level, 1=> increase, -1=> decrease
  # updateCaretPosition: if true, then the caret to moved to the end of the indentation
  def changeLineIndentation( lineNumber, direction, updateCaretPosition )
		indentation = [0, previousNonBlankLineIndentation(lineNumber) + direction*@editControl.indent() ].max()
		@editControl.setLineIndentation( lineNumber, indentation )
		@editControl.gotoPos( @editControl.lineIndentPosition( lineNumber ) ) if updateCaretPosition
  end

  def firstWordThatIsKeyWordAndNoEndOnLine( lineNumber )
	  keyWords = lineKeyWords( lineNumber )
	  if !keyWords.empty?
			keyWord = keyWords[0]
			keyWordIndex = keyWord[1]
			if @editControl.getLine( lineNumber )[0..keyWordIndex].chop() =~ /^\s*$/
				for keyWord in keyWords[1..-1]
				  if keyWord[0] == 'end'
				    return ''
					end
				end
				return keyWord[0]
			end
		end
		return ''
  end

  # Retreives the keywords of the specified line
  def lineKeyWords( lineNumber )
	  keyWords = Array.new
    return keyWords if lineNumber < 0
		styledText = @editControl.styledTextLineRange( lineNumber, lineNumber+1 )
	  text = styledText[0]
	  styles = styledText[1]
	  startingIndex = 0
		while startingIndex < styles.length()
		  index = styles[startingIndex..-1].index( Scintilla::Constant::SCE_P_WORD )
		  if index == nil
		    break
			end
      index += startingIndex
			endIndex = styles[index..-1].indexNotOf( Scintilla::Constant::SCE_P_WORD )
      endIndex += index
			keyWords.push( Array.new  << text[index..endIndex-1] << index )
			startingIndex = endIndex
		end
		keyWords
  end
end




