#unit test for BasicFoldDocument

require 'test/unit'
require 'TestBasicFoldDocument'

class FolderBuilder
  attr_reader :folders
  
  def initialize( folders = BasicFoldDocument.new() )
    @folders = folders
    @currentLine = 0
    @indentLevel = 0
  end
  
  def addFoldPoints( count=1 )
    while count > 0
      setNextLine( @indentLevel, FoldDocument::FOLD_POINT )
      count -= 1
      @indentLevel += 1
    end
    self
  end
  
  def addBlankLines( count=1 )
    while count > 0
      setNextLine( @indentLevel, FoldDocument::BLANK_LINE )
      count -= 1
    end
    self
  end
  
  def addLines( count=1 )
    while count > 0
      setNextLine( @indentLevel )
      count -= 1
    end
    self
  end
  
  def unindent( count=1 )
    @indentLevel -= count
    self
  end
  
  def indent( count=1 )
    @indentLevel += count
    self
  end
  
  private
  
  def setNextLine( level, flag = FoldDocument::NONE )
    @folders.lineCount = @currentLine+1
    @folders.setFoldLevelAndFlag( @currentLine, level, flag )
    @currentLine += 1
  end
end


class TestFoldDocument < TestBasicFoldDocument
	def set_up
    super
    @flagFolders = BasicFoldDocument.new( 3 )
    @flagFolders.setFoldFlag( 0, FoldDocument::FOLD_POINT )
    @flagFolders.setFoldFlag( 1, FoldDocument::BLANK_LINE )
    @flagFolders.setFoldFlag( 2, FoldDocument::NONE )
    setUpFolder1()
	end
	
	def tear_down
    super
    @flagFolders = nil
    @folders1 = nil
	end
  
  def setUpFolder1
    builder = FolderBuilder.new()
    #00 W
    #01 W
    #02 F
    #03  .
    #04  F
    #05   F
    #06    .
    #07    .
    #08   W
    #09   F
    #10    .
    #11   .
    builder.addBlankLines( 2 )
    builder.addFoldPoints()
    builder.addLines()
    builder.addFoldPoints( 2 )
    builder.addLines( 2 )
    builder.unindent()
    builder.addBlankLines()
    builder.addFoldPoints()
    builder.addLines()
    builder.unindent()
    builder.addLines()
    @folders1 = builder.folders
  end
	
	def testBlankLine?
    assert( !@flagFolders.blankLine?( 0 ) )
    assert( @flagFolders.blankLine?( 1 ) )
    assert( !@flagFolders.blankLine?( 2 ) )
	end
  
  def testFoldPoint?
    assert( @flagFolders.foldPoint?( 0 ) )
    assert( !@flagFolders.foldPoint?( 1 ) )
    assert( !@flagFolders.foldPoint?( 2 ) )
  end
  
  def testFindFoldPointLineFor
    assert_equal( -1, @folders1.findFoldPointLineFor( -5 ) )
    assert_equal( -1, @folders1.findFoldPointLineFor( 0 ) )
    assert_equal( -1, @folders1.findFoldPointLineFor( 1 ) )
    assert_equal( 2, @folders1.findFoldPointLineFor( 2 ) )
    assert_equal( 2, @folders1.findFoldPointLineFor( 3 ) )
    assert_equal( 4, @folders1.findFoldPointLineFor( 4 ) )
    assert_equal( 5, @folders1.findFoldPointLineFor( 5 ) )
    assert_equal( 4, @folders1.findFoldPointLineFor( 8 ) )
    assert_equal( 4, @folders1.findFoldPointLineFor( 11 ) )
    assert_equal( @folders1.findFoldPointLineFor( @folders1.lineCount-1 ), 
                  @folders1.findFoldPointLineFor( 12 ) )
  end

  def testFindFoldPointRange
    assert_nil( @folders1.findFoldPointRange( -1 ), 
                "nil should be returned for invalid line number" )
    assert_nil( @folders1.findFoldPointRange( 0 ), 
                "nil should be return fold non fold point" )
    assert_nil( @folders1.findFoldPointRange( 12 ), 
                "nil should be returned for invalid line number" )
    assert_nil( @folders1.findFoldPointRange( 11 ), 
                "nil should be return fold non fold point" )
    assert_equal( 9..10, @folders1.findFoldPointRange( 9 ) )
    assert_equal( 5..7, @folders1.findFoldPointRange( 5 ) )
    assert_equal( 4..11, @folders1.findFoldPointRange( 4 ) )
    assert_equal( 2..11, @folders1.findFoldPointRange( 2 ) )
  end
  
end



