#unit test for Observable

require 'test/unit'
require 'Observable'

class MockObserved
	include Observable
	
	def signalEventNameChanged( newName )
		signalEvent( :nameChanged, newName )
	end
	
	def signalEventSizeChanged( newWidth, newHeight )
		signalEvent( :sizeChanged, newWidth, newHeight )
	end
end

class TestObservable < Test::Unit::TestCase
	def set_up
		@observed = MockObserved.new
	end
	
	def tear_down
		@observed = nil
	end
	
	def testNoListener
		@observed.signalEventNameChanged( 'lain' )
		@observed.signalEventSizeChanged( 37, 12 )
	end
	
	def testOneListener
		called = false
		@observed.addListener( :nameChanged ) { |name|
				called = true
				assert_equal( 'gaia', name )
			}
		@observed.signalEventNameChanged( 'gaia' )
		assert( called, "listener was not called" )
	end
	
	def testTwoListeners
		called = false
		called2 = false
		@observed.addListener( :nameChanged ) { |name|
				called = true
				assert_equal( 'lain', name )
			}
		@observed.addListener( :nameChanged ) { |name|
				called2 = true
				assert_equal( 'lain', name )
			}
		@observed.signalEventNameChanged( 'lain' )
		
		assert( called, "listener was not called" )
		assert( called2, "listener2 was not called" )
	end
	
	def testDifferentEventTypes
		called = false
		called2 = false
		@observed.addListener( :nameChanged ) { |name|
				called = true
			}
		@observed.addListener( :sizeChanged ) { |width, height|
				called2 = true
				assert_equal( 37, width )
				assert_equal( 19, height )
			}
		@observed.signalEventNameChanged( 'lain' )
		@observed.signalEventSizeChanged( 37, 19 )
		
		assert( called, "listener was not called for event :nameChanged" )
		assert( called2, "listener2 was not called for event :sizeChanged" )
	end
	
	def testRemoveListener
		called = false
		listener = proc { |name|
				called = true
			}
		@observed.addListener( :nameChanged, &listener )
#		@observed.removeListener( :nameChanged, listener )
		@observed.signalEventNameChanged( 'lain' )
		assert( !called, "listener was called after being removed" )
	end
	
end













