# Base class for generated class Scintilla::EditControl
# handle type conversion and send message to the edit control thanks to a ruby extension

module Scintilla

	class EditControlBase
	
		def initialize( editorWindowHandle )
			if not editorWindowHandle.kind_of?( Integer )
				raise ArgumentError, "#{editorWindowHandle} is not a window handle (Integer)"
			end
			@editorWindowHandle = editorWindowHandle
		end
    
    def to_s()
      "EditControl<handle=%d>" % @editorWindowHandle
    end
	
		def asInt( value )
			raise ArgumentError, "#{value} is not a numeric" unless value.kind_of?( Numeric )
			value.to_i()
		end
	
		def asBoolean( value )
			value ? true : false
		end
	
		def asString( value )
			value.to_s()
		end
	
		def asColor( value )
			#raise ArgumentError, "#{value} is not a Color" unless value.kind_of?( Color )
			value.to_i()
		end
		
		def asModifiers( value )
			if value.kind_of?( Modifiers )
				value.to_i()
			else
				raise ArgumentError, "#{value} is nor a Numeric nor a Modifiers" unless value.kind_of?( Numeric )
				value.to_i()
			end
		end
	
		# Function added by the extension:
		# def sendEditorMessage( code, wParam, lParam )
		# => send a message to the editor window, and returns result value.
	
		# Event are received through object of class 'ScintillaNotification'.
		# A reader accessor is available for each field. It has the same name
		# the C structure field name:
		#
		# notification.eventType() returns the type of the event (SCN_XXX)
		# notification.ch() return a string containing the character ch (length=1)
		#
		#
		# WARNING: only the field compatible with the event type should be accessed!!!
		#
		# struct SCNotification {
		# 	struct NotifyHeader nmhdr;
		# 	int position;	// SCN_STYLENEEDED, SCN_MODIFIED, SCN_DWELLSTART, SCN_DWELLEND
		# 	int ch;		// SCN_CHARADDED, SCN_KEY
		# 	int modifiers;	// SCN_KEY
		# 	int modificationType;	// SCN_MODIFIED
		# 	const char *text;	// SCN_MODIFIED
		# 	int length;		// SCN_MODIFIED
		# 	int linesAdded;	// SCN_MODIFIED
		# 	int message;	// SCN_MACRORECORD
		# 	uptr_t wParam;	// SCN_MACRORECORD
		# 	sptr_t lParam;	// SCN_MACRORECORD
		# 	int line;		// SCN_MODIFIED
		# 	int foldLevelNow;	// SCN_MODIFIED
		# 	int foldLevelPrev;	// SCN_MODIFIED
		# 	int margin;		// SCN_MARGINCLICK
		# 	int listType;	// SCN_USERLISTSELECTION
		# 	int x;			// SCN_DWELLSTART, SCN_DWELLEND
		# 	int y;		// SCN_DWELLSTART, SCN_DWELLEND
		# };
		def scintillaEvent( notification )
		end
	
	
		# Returns the current line number
		def currentLineNumber
			lineFromPosition( currentPos() )
		end
	
		# Adds a text at the current position
		def addText( text )
			addTextMessage( text.length, text )
		end
	
		def replaceTarget( text )
			replaceTargetMessage( text.length(), text )
		end
		
		def replaceTextRange( from, to, text )
			self.targetStart = from
			self.targetEnd = to
			replaceTarget( text )
		end
	
		#fun int GetTextRange=2162(, textrange tr)
		# Retrieve a range of text.
		# Returns text including position from, to position to excluded.
		def textRange( from, to )
			sendEditorMessage( 2162, asInt(from), asInt(to) )
		end
	
		#fun void AssignCmdKey=2070(keymod km, int msg)
		# When key+modifier combination km is pressed perform msg.
		def assignCommandKey( key, modifiers, command )
			sendEditorMessage( 2070, makeKeyModifiers(key,modifiers), command )
		end
	
		#fun void ClearCmdKey=2071(keymod km,)
		# When key+modifier combination km do nothing.
		def clearCommandKey( key, modifiers )
			sendEditorMessage( 2071, makeKeyModifiers(key,modifiers), nil )
		end
	
		def makeKeyModifiers( key, modifiers )
			(asInt(key) & 0xffff) | ((asInt(modifiers) & 0xffff)<<16)
		end
	
		# Retrieve a range of text and its associated styles.
		# Returns a pair [text, [styles]]. First element contains a string with
		# the contains in the specified range (from,to). The second element is
		# an array of Integer, indicating the style of each character of text.
		#fun int GetStyledText=2015(, textrange tr)
		def styledText( from, to )
			sendEditorMessage( 2015, asInt( from ), asInt( to ) )
		end
	
		def styledTextLineRange( fromLineNumber, toLineNumber )
			styledText( positionFromLine( fromLineNumber ), positionFromLine( toLineNumber ) )
		end
	
	end
	
end


